from datetime import datetime
from typing import Union, List, Type, TypeVar, TypedDict, overload
from aiobotocore.client import AioBaseClient
from pydantic import parse_raw_as


class GetSecretValueResponse(TypedDict):
    ARN: str
    Name: str
    VersionId: str
    SecretBinary: bytes
    SecretString: str
    VersionStages: List[str]
    CreatedDate: datetime


SECRET_TYPE = TypeVar("SECRET_TYPE")


@overload
async def get_ssm_secret(ssm_client: AioBaseClient, secret_name: str, secret_type: Type[SECRET_TYPE]) -> SECRET_TYPE:
    pass


@overload
async def get_ssm_secret(ssm_client: AioBaseClient, secret_name: str, secret_type: Type = str) -> str:
    pass


async def get_ssm_secret(
    ssm_client: AioBaseClient,
    secret_name: str,
    secret_type: Type = str,
) -> Union[str, SECRET_TYPE]:
    """Gets an AWS Secrets Manager value

    example:
    ```
    @with_aws_client("secretsmanager")
    async def main(ssm_client: AioBaseClient) -> None:
        my_secret_str = await get_ssm_secret(ssm_client, "my_secret_key")
        my_secret_dict = await get_ssm_secret(ssm_client, "my_secret_key_dict", Dict[str, Any])
        my_secret_base_model = await get_ssm_secret(ssm_client, "my_secret_base_model", MyBaseModel)

    ```

    Args:
        ssm_client (AioBaseClient): aiobotocore client
        secret_name (str): Secret key
        secret_type (Type, optional): Secret value output. Defaults to str.

    Returns:
        Union[str, SECRET_TYPE]: Secret value
    """
    res: GetSecretValueResponse = await ssm_client.get_secret_value(SecretId=secret_name)
    secret_string = res["SecretString"]
    if secret_type == str:
        return secret_string
    return parse_raw_as(secret_type, secret_string)



from typing import Any, Callable, Coroutine, TypeVar

from clubbi_utils.logging import logger
from clubbi_utils.mysql.mysql_connector import MysqlConfig

F = TypeVar(
    "F",
    bound=Callable[..., Coroutine[Any, Any, Any]],
)
T = TypeVar("T")


def with_sql_engine(f: F) -> Callable[..., Coroutine[Any, Any, T]]:
    """this method is deprecated"""
    logger.warn("with_sql_egine is deprecated, use @MysqlConfig().with_engine()")
    return MysqlConfig().with_engine(f)



from typing import TypeVar, Optional


class ValueIsNoneError(ValueError):
    pass


T = TypeVar("T")


def none_assertion(v: Optional[T]) -> T:
    if v is None:
        raise ValueIsNoneError()
    return v



import asyncio
from typing import Any, Callable, TypeVar, Coroutine

from clubbi_utils.common.typing.lambda_context import LambdaContext

T = TypeVar('T')

LambdaHandler = Callable[[], Any]


def async_lambda(f: Callable[..., Coroutine[Any, Any, T]]) -> Callable[..., T]:
    def handle(event_data: Any, context: LambdaContext) -> T:
        return asyncio.run(f(event_data, context))

    return handle


def async_lambda_no_input(f: Callable[..., Coroutine[Any, Any, T]]) -> Callable[..., T]:
    def handle(_event_data: Any, _context: Any) -> T:
        return asyncio.run(f())

    return handle



import enum
from pydantic import BaseSettings
import logging


class LogLevelEnum(enum.Enum):
    fatal = "FATAL"
    error = "ERROR"
    warning = "WARNING"
    info = "INFO"
    debug = "DEBUG"
    notset = "NOTSET"

    def to_python_log_level(self) -> int:
        return {
            LogLevelEnum.fatal: logging.FATAL,
            LogLevelEnum.error: logging.ERROR,
            LogLevelEnum.warning: logging.WARNING,
            LogLevelEnum.info: logging.INFO,
            LogLevelEnum.debug: logging.DEBUG,
            LogLevelEnum.notset: logging.NOTSET,
        }.get(self, logging.INFO)


class LoggingSettings(BaseSettings):
    log_level: LogLevelEnum = LogLevelEnum.info


log_level = LoggingSettings().log_level.to_python_log_level()
if len(logging.getLogger().handlers) > 0:
    # The Lambda environment pre-configures a handler logging to stderr. If a handler is already configured,
    # `.basicConfig` does not execute. Thus we set the level directly.
    logging.getLogger().setLevel(log_level)
else:
    logging.basicConfig(level=log_level)

logger = logging.getLogger()



from typing import Any
from .logging import logger, LogLevelEnum
from clubbi_utils import json
from typing import Callable
import logging


class JsonLogger:
    def __init__(self, logger:logging.Logger):
        self.logger = logger
        setattr(self, "fatal", self._log(LogLevelEnum.fatal))
        setattr(self, "error", self._log(LogLevelEnum.error))
        setattr(self, "warning", self._log(LogLevelEnum.warning))
        setattr(self, "info", self._log(LogLevelEnum.info))
        setattr(self, "debug", self._log(LogLevelEnum.debug))

    def log(self, log_level: LogLevelEnum, workflow: str, message: str, **kwargs: Any) -> None:
        payload = dict(
            workflow=workflow,
            message=message,
            level=log_level,
            **kwargs,
        )
        self.logger.log(level=log_level.to_python_log_level(), msg=json.dumps(payload))

    def _log(self, log_level: LogLevelEnum) -> Callable:
        return lambda workflow, message, **kwargs: self.log(log_level, workflow, message, **kwargs)

    def fatal(self, workflow: str, message: str, **kwargs: Any) -> None:
        pass

    def error(self, workflow: str, message: str, **kwargs: Any) -> None:
        pass

    def warning(self, workflow: str, message: str, **kwargs: Any) -> None:
        pass

    def info(self, workflow: str, message: str, **kwargs: Any) -> None:
        pass

    def debug(self, workflow: str, message: str, **kwargs: Any) -> None:
        pass


jlogger = JsonLogger(logger)



# Docs: https://docs.aws.amazon.com/lambda/latest/dg/with-sqs-example.html#with-sqs-create-test-function
from typing import Dict, List, TypedDict


class AwsSqsRecord(TypedDict):
    messageId: str
    body: str
    eventSource: str


class AwsSqsEvent(TypedDict):
    Records: List[AwsSqsRecord]



from typing import Callable, Optional
from unittest import TestCase

from clubbi_utils.operators import none_coalesce, none_assertion, ValueIsNoneError


class TestOperators(TestCase):

    def test_none_coalesce(self):
        ans = none_coalesce(1, 2, 3, default=4)
        self.assertEqual(ans, 1)
    
    def test_none_coalesce_with_single_none(self):
        ans = none_coalesce(None, 2, 3, default=4)
        self.assertEqual(ans, 2)
    
    def test_none_coalesce_with_default_non_none(self):
        ans = none_coalesce(None, None, None, default="a")
        self.assertEqual(ans, "a")
    
    def test_none_coalesce_with_default_none(self):
        ans = none_coalesce(None, None, None, default=None)
        self.assertIsNone(ans)
    
    def test_none_assertion(self):
        f: Callable[[Optional[int]], int] = lambda v: none_assertion(v)+1
        ans = f(1)
        self.assertEqual(ans, 2)

        with self.assertRaises(ValueIsNoneError):
            f(None)
        




from typing import Optional, TypedDict, Dict, List

__all__ = [
    "AwsHttpEvent",
    "AwsHttpResponse"
]

# Aws payload format version 2.0 (simplified)
# REF.: https://docs.aws.amazon.com/apigateway/latest/developerguide/http-api-develop-integrations-lambda.html


class AwsHttpEvent(TypedDict, total=False):
    body: str
    headers: Dict[str, str]
    queryStringParameters: Dict[str, str]
    pathParameters: Dict[str, str]


class AwsHttpResponse(TypedDict, total=False):
    statusCode: int
    body: Optional[str]
    headers: Optional[Dict[str, str]]
    cookies: Optional[List[str]]
    isBase64Encoded: Optional[bool]



from typing import TypeVar, Callable, Any, Coroutine

from clubbi_utils.sqlalchemy.connector import SqlAlchemyConfig

F = TypeVar(
    "F",
    bound=Callable[..., Coroutine[Any, Any, Any]],
)
T = TypeVar("T")


def with_sqlalchemy_engine(f: F) -> Callable[..., Coroutine[Any, Any, T]]:
    # config: Optional[MysqlConfig] = None
    # def _dec(f: F) -> Callable[..., Coroutine[Any, Any, T]]:
    async def wrapper(*args: Any, **kwargs: dict) -> Any:
        db = SqlAlchemyConfig().create_engine()
        try:
            r = await f(db, *args, **kwargs)
            return r
        finally:
            await db.dispose()

    return wrapper
